@@ -18,6 +18,7 @@
  */
 package org.apache.openjpa.jdbc.sql;
 
+import java.lang.reflect.Field;
 import java.math.BigDecimal;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
@@ -25,9 +26,9 @@ import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.Types;
 import java.util.Arrays;
+import java.util.Locale;
 
 import org.apache.commons.lang.StringUtils;
-import org.hsqldb.Trace;
 import org.apache.openjpa.jdbc.identifier.DBIdentifier;
 import org.apache.openjpa.jdbc.kernel.exps.FilterValue;
 import org.apache.openjpa.jdbc.schema.Column;
@@ -50,6 +51,8 @@ public class HSQLDictionary extends DBDictionary {
     public boolean cacheTables = false;
 
     private int dbMajorVersion;
+    private int dbMinorVersion;
+    private int violation_of_unique_index_or_constraint;
 
     private SQLBuffer _oneBuffer = new SQLBuffer(this).append("1");
 
@@ -95,6 +98,8 @@ public class HSQLDictionary extends DBDictionary {
         fixedSizeTypeNameSet.addAll(Arrays.asList(new String[]{
             "TEXT"
         }));
+        fixedSizeTypeNameSet.remove("NUMERIC");
+        fixedSizeTypeNameSet.remove("DECIMAL");
     }
 
     /**
@@ -114,6 +119,28 @@ public class HSQLDictionary extends DBDictionary {
             // but doesn't support it for columns references ("schema.table.column")
             useSchemaName = false;
         }
+        if (dbMajorVersion > 1 && dbMinorVersion > 0) {
+            nextSequenceQuery += " LIMIT 1";
+        }
+        String packageName;
+        String fieldName;
+        if (dbMajorVersion > 1) {
+            // default value for "X_23505"
+            violation_of_unique_index_or_constraint = 104;
+            packageName = "org.hsqldb.error.ErrorCode";
+            fieldName = "X_23505";
+        } else {
+            // default value for "VIOLATION_OF_UNIQUE_INDEX"
+            violation_of_unique_index_or_constraint = 9; 
+            packageName = "org.hsqldb.Trace";
+            fieldName = "VIOLATION_OF_UNIQUE_INDEX";
+        }
+        try {
+            Class<?> cls = Class.forName(packageName);
+            Field fld = cls.getField(fieldName);
+            violation_of_unique_index_or_constraint = fld.getInt(null);
+        } catch (Exception e) {
+        }
     }
 
     /**
@@ -126,11 +153,13 @@ public class HSQLDictionary extends DBDictionary {
 
         if (isJDBC3) {
             dbMajorVersion = metaData.getDatabaseMajorVersion();
+            dbMinorVersion = metaData.getDatabaseMinorVersion();
         } else {
             // String is like "2.0.0"
             String productVersion = metaData.getDatabaseProductVersion();
             String[] version = productVersion.split("\\.") ;
             dbMajorVersion = Integer.parseInt(version[0]) ;
+            dbMinorVersion = Integer.parseInt(version[1]);
         }
     }
 
@@ -222,7 +251,7 @@ public class HSQLDictionary extends DBDictionary {
     }
 
     public boolean isSystemIndex(String name, Table table) {
-        return name.toUpperCase().startsWith("SYS_");
+        return name.toUpperCase(Locale.ENGLISH).startsWith("SYS_");
     }
 
     @Override
@@ -230,7 +259,7 @@ public class HSQLDictionary extends DBDictionary {
         if (DBIdentifier.isNull(name)) {
             return false;
         }
-        return name.getName().toUpperCase().startsWith("SYS_");
+        return name.getName().toUpperCase(Locale.ENGLISH).startsWith("SYS_");
     }
 
     @Override
@@ -375,7 +404,7 @@ public class HSQLDictionary extends DBDictionary {
         Object failed) {
         OpenJPAException ke = super.newStoreException(msg, causes, failed);
         if (ke instanceof ReferentialIntegrityException
-            && causes[0].getErrorCode() == -Trace.VIOLATION_OF_UNIQUE_INDEX) {
+            && causes[0].getErrorCode() == -violation_of_unique_index_or_constraint) {
             ((ReferentialIntegrityException) ke).setIntegrityViolation
                 (ReferentialIntegrityException.IV_UNIQUE);
         }
