@@ -44,6 +44,7 @@ import org.apache.openjpa.enhance.PersistenceCapable;
 import org.apache.openjpa.event.LifecycleEventManager;
 import org.apache.openjpa.lib.conf.Configurable;
 import org.apache.openjpa.lib.conf.Configuration;
+import org.apache.openjpa.lib.conf.Configurations;
 import org.apache.openjpa.lib.log.Log;
 import org.apache.openjpa.lib.util.Closeable;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
@@ -1900,7 +1901,7 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
             _metamodel.put(entity, m2cls);
             return m2cls;
         } catch (Throwable t) {
-            if (_log.isInfoEnabled())
+            if (_log.isTraceEnabled())
                 _log.warn(_loc.get("meta-no-model", m2, entity, t));
         }
         return null;
@@ -1999,17 +2000,16 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
     private QueryMetaData getQueryMetaDataInternal(Class<?> cls, String name, ClassLoader envLoader) {
         if (name == null)
             return null;
-        QueryMetaData qm = null;
-        if (cls == null) {
-            qm = searchQueryMetaDataByName(name);
-            if (qm != null)
-                return qm;
-        }
+
         // check cache
-        qm = (QueryMetaData) _queries.get(name);
+        QueryMetaData qm = (QueryMetaData) _queries.get(name);
         if (qm != null)
             return qm;
 
+        // see if factory can figure out a scope for this query
+        if (cls == null)
+            cls = _factory.getQueryScope(name, envLoader);
+        
         // get metadata for class, which will find queries in metadata file
         if (cls != null && getMetaData(cls, envLoader, false) != null) {
             qm = _queries.get(name);
@@ -2018,13 +2018,9 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
         }
         if ((_sourceMode & MODE_QUERY) == 0)
             return null;
-
-        // see if factory can figure out a scope for this query
-        if (cls == null)
-            cls = _factory.getQueryScope(name, envLoader);
-
+        
         // not in cache; load
-        _factory.load(cls, MODE_QUERY, envLoader);
+        _factory.load(cls, MODE_QUERY , envLoader);
         return _queries.get(name);
     }
 
@@ -2040,11 +2036,12 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
             return (QueryMetaData[]) _queries.values().toArray(new QueryMetaData[_queries.size()]);
         }
     }
+    
 
     public QueryMetaData getCachedQueryMetaData(Class<?> cls, String name) {
         return getCachedQueryMetaData(name);
     }
-    
+
     /**
      * Return the cached query metadata for the given name.
      */
@@ -2407,7 +2404,6 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
     /**
      * Return XML metadata for a given field metadata
      * 
-     * @param fmd
      * @return XML metadata
      */
     public XMLMetaData getXMLMetaData(Class<?> cls) {
@@ -2466,8 +2462,6 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
     /**
      * Create a new xml class metadata
      * 
-     * @param type
-     * @param name
      * @return a XMLClassMetaData
      */
     protected XMLClassMetaData newXMLClassMetaData(Class<?> type) {
@@ -2477,17 +2471,17 @@ public class MetaDataRepository implements PCRegistry.RegisterClassListener, Con
     /**
      * Create a new xml field meta, add it to the fieldMap in the given xml class metadata
      * 
-     * @param type
-     * @param name
-     * @param meta
      * @return a XMLFieldMetaData
      */
     public XMLFieldMetaData newXMLFieldMetaData(Class<?> type, String name) {
         return new XMLFieldMetaData(type, name);
     }
 
-    public static boolean needsPreload(Options o) {
-        if (o.getBooleanProperty(PRELOAD_STR) == true) {
+    public static boolean needsPreload(OpenJPAConfiguration conf) {
+        if (conf == null)
+            return false;
+        Options o = Configurations.parseProperties(Configurations.getProperties(conf.getMetaDataRepository()));
+        if (o.getBooleanProperty(PRELOAD_STR) == true || o.getBooleanProperty(PRELOAD_STR.toLowerCase()) == true) {
             return true;
         }
         return false;
