@@ -23,6 +23,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -43,7 +44,6 @@ import javax.persistence.criteria.Selection;
 import javax.persistence.criteria.Subquery;
 import javax.persistence.metamodel.EntityType;
 
-import org.apache.commons.collections.map.LinkedMap;
 import org.apache.openjpa.kernel.StoreQuery;
 import org.apache.openjpa.kernel.exps.Context;
 import org.apache.openjpa.kernel.exps.ExpressionFactory;
@@ -95,8 +95,13 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
     private Map<Selection<?>,Value> _rootVariables = new HashMap<Selection<?>, Value>();
     
     // SubqueryContext
-    private Stack<Context> _contexts = null;
-    
+    private ThreadLocal<Stack<Context>> _contexts = new  ThreadLocal<Stack<Context>>(){
+        @Override
+        protected Stack<Context> initialValue() {
+            return new Stack<Context>();
+        }
+    };
+
     public CriteriaQueryImpl(MetamodelImpl model, Class<T> resultClass) {
         this._model = model;
         this._resultClass = resultClass;
@@ -135,7 +140,7 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
      * Gets the stack of contexts used by this query.
      */
     Stack<Context> getContexts() {
-        return _contexts;
+        return _contexts.get();
     }
     
     /**
@@ -342,7 +347,7 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
 
     
     public <X> Root<X> from(Class<X> cls) {
-        EntityType<X> entity = _model.entity(cls);
+        EntityType<X> entity = _model.entityImpl(cls);
         if (entity == null)
             throw new IllegalArgumentException(_loc.get("root-non-entity", cls).getMessage());
         return from(entity);
@@ -412,10 +417,13 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
      * receiver with the help of the given {@link ExpressionFactory}.
      */
     QueryExpressions getQueryExpressions(ExpressionFactory factory) {
-        _contexts = new Stack<Context>();
         Context context = new Context(null, null, null);
-        _contexts.push(context);
-        return new CriteriaExpressionBuilder().getQueryExpressions(factory, this);
+        _contexts.get().push(context);
+        try {
+            return new CriteriaExpressionBuilder().getQueryExpressions(factory, this);
+        }finally{
+            _contexts.remove();
+        }
     }    
     
     public void assertRoot() {
@@ -432,7 +440,7 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
     // SubqueryContext
     //
     void setContexts(Stack<Context> contexts) {
-        _contexts = contexts;
+        _contexts.set(contexts);
     }
     
     /**
@@ -461,7 +469,8 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
      * Gets the current context.
      */
     Context ctx() {
-        return _contexts == null || _contexts.isEmpty() ? null :  _contexts.peek();
+        Stack<Context> ctxt = _contexts.get();
+        return ctxt == null || ctxt.isEmpty() ? null :  ctxt.peek();
     }
     
     //
@@ -669,24 +678,25 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
         if (_where != null) {
             buffer.append(" WHERE ").append(_where.asValue(this));
         }
-        if (_orders != null) {
-            buffer.append(" ORDER BY ");
-            for (Order orderBy : getOrderList()) {
-                buffer.append(((CriteriaExpression)orderBy).asValue(this));
-            }
-        }
-        if (_groups != null) {
-            buffer.append(" GROUP BY ");
-            for (Expression<?> groupBy : getGroupList()) {
-                buffer.append(((CriteriaExpression)groupBy).asValue(this));
-            }
-        }
+        renderList(buffer, " ORDER BY ", getOrderList());
+        renderList(buffer, " GROUP BY ", getGroupList());
         if (_having != null) {
             buffer.append(" HAVING ");
             buffer.append(_having.asValue(this));
         }
     }
     
+    private void renderList(StringBuilder buffer, String clause, Collection<?> coll) {
+    	if (coll == null || coll.isEmpty())
+    		return;
+    	
+    	buffer.append(clause);
+    	for (Iterator<?> i = coll.iterator(); i.hasNext(); ) {
+    		buffer.append(((CriteriaExpression)i.next()).asValue(this));
+    		if (i.hasNext()) buffer.append(", ");
+    	}
+    }
+    
     private void renderJoins(StringBuilder buffer, Collection<Join<?,?>> joins) {
         if (joins == null) return;
         for (Join j : joins) {
@@ -726,6 +736,10 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
     }
     
     public boolean equals(Object other) {
+        if(other == null) {
+            return false;
+        }
+        
         if (toString().equals(other.toString()))
             return true;
         return false;
